@@ -6,10 +6,12 @@ import re
 import shlex
 import subprocess
 
+from clouds.aws.aws_utils import verify_aws_credentials
 from simple_logger.logger import get_logger
 
 
 LOGGER = get_logger(__name__)
+TIMEOUT_5MIN = 5 * 60
 
 
 class CommandExecuteError(Exception):
@@ -22,11 +24,16 @@ class NotLoggedInError(Exception):
 
 @contextlib.contextmanager
 def rosa_login_logout(env, token, aws_region, allowed_commands=None):
+    verify_aws_credentials()
     _allowed_commands = allowed_commands or parse_help()
     build_execute_command(
-        command=f"login --region {aws_region} {f'--env={env}' if env else ''} --token={token}",
+        command=f"login {f'--env={env}' if env else ''} --token={token}",
         allowed_commands=_allowed_commands,
+        aws_region=aws_region,
     )
+    if not is_logged_in(allowed_commands=_allowed_commands, aws_region=aws_region):
+        raise NotLoggedInError("Failed to login to AWS.")
+
     yield
     build_execute_command(command="logout", allowed_commands=_allowed_commands)
 
@@ -50,12 +57,13 @@ def is_logged_in(aws_region=None, allowed_commands=None):
         return False
 
 
-def execute_command(command):
+def execute_command(command, wait_timeout=TIMEOUT_5MIN):
     joined_command = " ".join(command)
     LOGGER.info(
-        f"Executing command: {re.sub(r'(--token=.* |--token=.*)', '--token=hashed-token ', joined_command)}"
+        f"Executing command: {re.sub(r'(--token=.* |--token=.*)', '--token=hashed-token', joined_command)}, "
+        f"waiting for {wait_timeout} seconds."
     )
-    res = subprocess.run(command, capture_output=True, text=True)
+    res = subprocess.run(command, capture_output=True, text=True, timeout=wait_timeout)
     if res.returncode != 0:
         raise CommandExecuteError(f"Failed to execute: {res.stderr}")
 
@@ -158,57 +166,85 @@ def parse_help(rosa_cmd="rosa"):
 
     for top_command in commands_dict.keys():
         _commands = get_available_commands(command=[rosa_cmd, top_command])
-        for command in _commands:
-            commands_dict[top_command][command] = {}
-            _commands = get_available_commands(command=[rosa_cmd, top_command, command])
-            if _commands:
-                for _command in _commands:
-                    commands_dict[top_command][command][_command] = {}
-                    commands_dict[top_command][command][_command][
+
+        if _commands:
+            # If top command has sub command
+            for command in _commands:
+                commands_dict[top_command][command] = {}
+                _commands = get_available_commands(
+                    command=[rosa_cmd, top_command, command]
+                )
+                if _commands:
+                    # If sub command has sub command
+                    for _command in _commands:
+                        commands_dict[top_command][command][_command] = {}
+                        commands_dict[top_command][command][_command][
+                            "json_output"
+                        ] = check_flag_in_flags(
+                            command_list=[rosa_cmd, top_command, _command],
+                            flag_str=output_flag_str,
+                        )
+                        commands_dict[top_command][command][_command][
+                            "auto_answer_yes"
+                        ] = check_flag_in_flags(
+                            command_list=[rosa_cmd, top_command, _command],
+                            flag_str=auto_answer_yes_str,
+                        )
+                        commands_dict[top_command][command][_command][
+                            "auto_mode"
+                        ] = check_flag_in_flags(
+                            command_list=[rosa_cmd, top_command, _command],
+                            flag_str=auto_mode_str,
+                        )
+                        commands_dict[top_command][command][_command][
+                            "region"
+                        ] = check_flag_in_flags(
+                            command_list=[rosa_cmd, top_command, _command],
+                            flag_str=region_str,
+                        )
+                else:
+                    # If sub command doesn't have sub command
+                    commands_dict[top_command][command][
                         "json_output"
                     ] = check_flag_in_flags(
-                        command_list=[rosa_cmd, top_command, _command],
+                        command_list=[rosa_cmd, top_command, command],
                         flag_str=output_flag_str,
                     )
-                    commands_dict[top_command][command][_command][
+                    commands_dict[top_command][command][
                         "auto_answer_yes"
                     ] = check_flag_in_flags(
-                        command_list=[rosa_cmd, top_command, _command],
+                        command_list=[rosa_cmd, top_command, command],
                         flag_str=auto_answer_yes_str,
                     )
-                    commands_dict[top_command][command][_command][
+                    commands_dict[top_command][command][
                         "auto_mode"
                     ] = check_flag_in_flags(
-                        command_list=[rosa_cmd, top_command, _command],
+                        command_list=[rosa_cmd, top_command, command],
                         flag_str=auto_mode_str,
                     )
-                    commands_dict[top_command][command][_command][
-                        "region"
-                    ] = check_flag_in_flags(
-                        command_list=[rosa_cmd, top_command, _command],
+                    commands_dict[top_command][command]["region"] = check_flag_in_flags(
+                        command_list=[rosa_cmd, top_command, command],
                         flag_str=region_str,
                     )
-            else:
-                commands_dict[top_command][command][
-                    "json_output"
-                ] = check_flag_in_flags(
-                    command_list=[rosa_cmd, top_command, command],
-                    flag_str=output_flag_str,
-                )
-                commands_dict[top_command][command][
-                    "auto_answer_yes"
-                ] = check_flag_in_flags(
-                    command_list=[rosa_cmd, top_command, command],
-                    flag_str=auto_answer_yes_str,
-                )
-                commands_dict[top_command][command]["auto_mode"] = check_flag_in_flags(
-                    command_list=[rosa_cmd, top_command, command],
-                    flag_str=auto_mode_str,
-                )
-                commands_dict[top_command][command]["region"] = check_flag_in_flags(
-                    command_list=[rosa_cmd, top_command, command],
-                    flag_str=region_str,
-                )
+
+        else:
+            # If top command doesn't have sub command
+            commands_dict[top_command]["json_output"] = check_flag_in_flags(
+                command_list=[rosa_cmd, top_command],
+                flag_str=output_flag_str,
+            )
+            commands_dict[top_command]["auto_answer_yes"] = check_flag_in_flags(
+                command_list=[rosa_cmd, top_command],
+                flag_str=auto_answer_yes_str,
+            )
+            commands_dict[top_command]["auto_mode"] = check_flag_in_flags(
+                command_list=[rosa_cmd, top_command],
+                flag_str=auto_mode_str,
+            )
+            commands_dict[top_command]["region"] = check_flag_in_flags(
+                command_list=[rosa_cmd, top_command],
+                flag_str=region_str,
+            )
 
     return commands_dict
 
